"""
========================================================================
|                                                                      |
|                       SUSYAI EXAMPLE SCRIPT 16:                      |
|                             FULL EXAMPLE                             |
|   ----------------------------------------------------------------   |
|                                                                      |
|   SUSY-AI is a powerful, yet simple interface between SUSY           |
|   data and the sklearn Python class. With the classifier that        |
|   comes with SUSY-AI, it allows for predicting of exclusion of a     |
|   datapoint in pMSSM parameterspace via sklearn using a              |
|   pre-trained classifier in only a couple of commands.               |
|                                                                      |
|   Full documentation, and the most recent version of the software    |
|   can be found on the project website                                |
|                                                                      |
|       https://susyai.hepforge.org/                                   |
|                                                                      |
|   If you are using this script scientifically or commercially,       |
|   please reference the following paper                               |
|                                                                      |
|       Sascha Caron, Jong Soo Kim, Krzysztof Rolbiecki, Roberto       |
|       Ruiz de Austri, Bob Stienen, 'The BSM-AI project: SUSY-AI -    |
|       Generalizing LHC limits on Supersymmetry with Machine          |
|       Learning'. arXiv: 1605.02797 [hep-ph]                          |
|                                                                      |
|   For questions about the script, contact                            |
|                                                                      |
|       Bob Stienen <b.stienen@student.ru.nl>                          |
|                                                                      |
========================================================================
"""

from susyai import susyai
import numpy as np
from os import popen

# Define files
folders = ['spectrumfiles']
files = []
for i in range(len(folders)):
	fs = popen('ls %s | grep "%s"' % (folders[i], '.slha')).readlines()
	for j in range(len(fs)):
		files.append(folders[i]+'/'+fs[j].rstrip())

# Define coorindate selection function
# This function must take a single PYSLHA DOC object as input argument
#   and return a (19,)-shaped NUMPY NDARRAY, containing the coordinates
def csel(spectrum):
	coords = np.zeros(19)
	coords[0] = spectrum.blocks['MSOFT'][1]           # M_1
	coords[1] = spectrum.blocks['MSOFT'][2]           # M_2
	coords[2] = spectrum.blocks['MSOFT'][3]           # M_3
	coords[3] = spectrum.blocks['MSOFT'][31]          # mL1
	coords[4] = spectrum.blocks['MSOFT'][33]          # mL3
	coords[5] = spectrum.blocks['MSOFT'][34]          # me1
	coords[6] = spectrum.blocks['MSOFT'][36]          # me3
	coords[7] = spectrum.blocks['MSOFT'][41]          # mQ1
	coords[8] = spectrum.blocks['MSOFT'][43]          # mQt
	coords[9] = spectrum.blocks['MSOFT'][44]          # mu1
	coords[10] = spectrum.blocks['MSOFT'][46]         # mtR
	coords[11] = spectrum.blocks['MSOFT'][47]         # md1
	coords[12] = spectrum.blocks['MSOFT'][49]         # mbR
	coords[13] = spectrum.blocks['AU'][(3,3)]         # At
	coords[14] = spectrum.blocks['AD'][(3,3)]         # Ab
	coords[15] = spectrum.blocks['AE'][(3,3)]         # Atau
	coords[16] = spectrum.blocks['HMIX'][1]           # mu
	coords[17] = spectrum.blocks['HMIX'][4]           # mA2
	coords[18] = spectrum.blocks['HMIX'][2]           # tanbeta
	return coords


# Define xtra selection function with dict_to_matrix method
# The xtra selector function must take two arguments:
#   a PYSLHA DOC object and
#   a string containing the path to the file
# The xtra selector has to return the data in the form of a NUMPY NDARRAY of shape (N,)
#   where N can be any integer the user wants, as long as it does not change in a single run
# This particular function selects the left and righthanded stop particles
def xsel(spectrum, filepath):
	staumatrix = np.linalg.inv(susyai.dict_to_matrix(spectrum.blocks['STAUMIX']))
	stau = np.array([spectrum.blocks['MASS'][1000015], spectrum.blocks['MASS'][2000015]])
	staus = np.abs(np.dot(staumatrix, stau))
	return staus[:2]


# Creating susyai object
sa = susyai()

# Set the location of the pickle to a valid location before testing
# Loading before prediction removes the need for it during prediction, thus speeding it up
sa.set_pickle_location("pmssm.pkl")
sa.load_pickle()

# Set coordinate selection to SLHA-1 standard
sa.set_coordinate_selector(csel)

# Set xtra selection to user defined function
sa.set_xtra_selector(xsel)

# Set id selection to 'filename'
sa.set_id_selector('filename')

# Set mapping of coordinates (default: False)
sa.set_map_outsiders(True)

# Prediction with a minimal confidence of 0.95
# Note that variable COORDS now containes the mapped coordinates!
clas, pred, conf, coords, xtras, ids = sa.predict_files(files, min_confidence = 0.95)


print("classification: ",clas)
print("prediction:     ",pred)
print("confidence:     ",conf)
print("xtras:          ",xtras)
print("ids:            ",ids)
